%% step 1 Morphologically dealing with the broken parts of the remote sensing images
[data,corLS]=geotiffread('\');                                                            % Input river data in TIF format \means the path of the TIF
data(data~=1)=0;                                                                 
n=1;                                                                                      % parameter n means Number of bridge function executions.       
data1 = bwmorph(data,'bridge',n);
data1 = uint8(data1);
info=geotiffinfo('\');
geotiffwrite('\',data1,corLS,'GeoKeyDirectoryTag',info.GeoTIFFTags.GeoKeyDirectoryTag);   % Output the result of the river connecting in TIF format 
clear all; 
% Morphological processing of tif format products, Filling the hole and disconnected stream
[data,corLS]=geotiffread('\');     
data(data~=1)=0;
n=1;                                                                                      % parameter n means Number of 'fill' function executions.                                          
data1 = bwmorph(data,'fill',n);                                                  
data1 = uint8(data1);
info=geotiffinfo('\');
geotiffwrite('\',data1,corLS,'GeoKeyDirectoryTag',info.GeoTIFFTags.GeoKeyDirectoryTag);
clear all;
[data,corLS]=geotiffread('\');     
data(data~=1)=0;                                                      
n=1;                                                                 
data1 = bwmorph(data,'bridge',n);
data1 = uint8(data1);
info=geotiffinfo('\');
geotiffwrite('\',data1,corLS,'GeoKeyDirectoryTag',info.GeoTIFFTags.GeoKeyDirectoryTag);  
%% step 2 Extraction process of river skeleton line or center line
[data,corLS]=geotiffread('\');   
data(data~=1)=0; 
data1 = bwmorph(data,'thin',Inf);                                                         % function 'thin' will generate the river skeleton
num_XH=length(find((data1)==1));            
data2=int8(data1);
num_XH1=length(find((data2)==1));            
info=geotiffinfo('\');
geotiffwrite('\',data2,corLS,'GeoKeyDirectoryTag',info.GeoTIFFTags.GeoKeyDirectoryTag);
clear data corLS data1 data2 info num_XH num_XH1;
% Morphological algorithms deal with interrupted streams
[data4,corLS2]=geotiffread('\');      
nn=1;                                                                                      % function 'bridge' will ensure the connectivity of the river skeleton    
data5 = bwmorph(data4,'bridge',nn);
data5 = uint8(data5);
info2=geotiffinfo('\');
geotiffwrite('\',data5,corLS2,'GeoKeyDirectoryTag',info2.GeoTIFFTags.GeoKeyDirectoryTag);  
%% step3 Calculation of virtual hydrographic station section of the preparatory work
Lat=xlsread('\',1);       % input latitude coordinates in the format xxx.xxx and make sure you have complete river centerline discrete point and virtual hydrographic station coordinates.Getting these from arcgis.
Lon=xlsread('\',2);       % longitude coordinates  
Lat1=xlsread('\',3);      % latitude coordinates   
Lon1=xlsread('\',4);      % longitude coordinates
para=xlsread('\',5);      % Storage parameters to Locate the row & Column number of the virtual hydrological station such as Lower left datum coordinates
Left=para(1,1) ;              
Bott=para(1,2) ;              
tif=para(1,3);                
[x,y,utmzone] = deg2utm(Lat,Lon);
[x1,y1,utmzone1] =  deg2utm(Lat1,Lon1);
utmzone_ls = utmzone(:,1:2);
utmzone1_ls = utmzone1(:,1:2);
utm_ls = str2num(utmzone_ls);
utm1_ls =  str2num(utmzone1_ls);
Row_1 = (y-Bott)./tif;
Col_1 = (x-Left)./tif;
Row =fix(Row_1)+1;      
Col =fix(Col_1)+1;
Row_2 = (y1-Bott)./tif;
Col_2 = (x1-Left)./tif;
Row1 =fix(Row_2)+1;           
Col1 =fix(Col_2)+1;      
% Output the projected coordinates in order of x, y, row ,column and UTM code 
[m,~]=size(x);
ZB_UTM=zeros(m,6,'double'); 
num=1:m;                     
ZB_UTM(:,1)=num;
ZB_UTM(:,2)=x;
ZB_UTM(:,3)=y;
ZB_UTM(:,4)=Row;
ZB_UTM(:,5)=Col;
ZB_UTM(:,6)=utm_ls;     
uu=unique(utm_ls);
[n,~]=size(uu);
utm_all=zeros(m,6,n,'double');   
kk=zeros(n,1);
for i = 1:n                   
   utm=uu(i);
   [utm1,~] = find(utm_ls==utm);
   [k,~]=size(utm1);
   kk(i)=k;
   utm11=ZB_UTM(utm1,:);
   utm_all(1:k,:,i)=utm11;
end
kk1=max(kk);                  
zb_utm=zeros(kk1,6*n,'double');
n1=1;                         
for i=1:n
   zb_utm(1:kk(i),n1:6*i)=utm_all(:,:,1); 
   n1=6*i+1;
end
clear utm_all;
[m,~]=size(x1);                
ZB_UTM1=zeros(m,6,'double');
num1=1:m;                    
ZB_UTM1(:,1)=num1;
ZB_UTM1(:,2)=x1;
ZB_UTM1(:,3)=y1;
ZB_UTM1(:,4)=Row1;
ZB_UTM1(:,5)=Col1;
ZB_UTM1(:,6)=utm1_ls;
uu1=unique(utm1_ls);
[n,~]=size(uu1);
utm1_all=zeros(m,6,n,'double');
kk2=zeros(n,1);
for i = 1:n
   utm2=uu1(i);
   [utm3,~] = find(utm1_ls==utm2);
   [k,~]=size(utm3);
    kk2(i)=k;
   utm21=ZB_UTM1(utm3,:);
   utm1_all(1:k,:,i)=utm21;
end
kk3=max(kk2);                 
zb_utm1=zeros(kk3,6*n,'double');
n1=1;                        
for i=1:n
   zb_utm1(1:kk2(i),n1:6*i)=utm1_all(:,:,1); 
   n1=6*i+1;
end
clear utm1_all;
dlmwrite('\',zb_utm,'delimiter',',','precision','%12.5f');
dlmwrite('\',zb_utm1,'delimiter',',','precision','%12.5f');
%% step4 The river width calculation
% It needs two points adjacent to the virtual hydrological station on the Central Line and the virtual hydrological station 
% in total three points to determine the river width at every virtual station.
flow1=xlsread('\',1); % Read the coordinates of discrete grid center point of river skeleton line, and its access format is X and Y. 
flow2=xlsread('\',2); % flow1&2 Contains serial number, abscissa, ordinate, row number, column number in total of 5 kinds of source data.
[m1,~]=size(flow1); 
[m2,~]=size(flow2); 
F1 = flow1(:,4:5);   
F2 = flow2(:,4:5);
X=zeros(m1,1,'double');
l=0;
for i = 1:m2                            
  A = F2(i,1);
  [x,~] = find(F1(:,1) ==A );
  [K1,~]=size(x);
  z=l+K1;
  w=1;
  for j=l+1:z
      X(j)=x(w);
      w=w+1;
  end
   l=sum(X~=0);
end                                   
X(X==0)=[];                            
[K1,~]=size(X);
X1=zeros(K1,3,'double');
X1(:,1)=X;
X1(:,2)=F1(X,1);
X1(:,3)=F1(X,2);
X2=zeros(m1,1,'double');
l=0;
for i = 1:m2                           
  A = F2(i,2);
  [x,~] = find(X1(:,3) ==A );
  [K1,~]=size(x);
  z=l+K1;
  w=1;
  for j=l+1:z
      X2(j)=x(w);
      w=w+1;
  end
   l=sum(X2~=0);
end                                    
X2(X2==0)=[];                          
FID=X1(X2,1);                         
KK=F1(FID,:);                          
c1=ismember(KK,F2,'rows');   
[c2,~]=find(c1==0);
FID(c2,:)=[];   
FID=unique(FID);
beg=max(FID);                          
en=min(FID);
[m,~]=size(FID);
n=(m-2)*2+(m-2)+2*2;                   
CZ=zeros(n,1,'double');
CZ(1)=en;
CZ(2)=en+1;
CZ(n)=beg;
CZ(n-1)=beg-1;                        
s=3;
for i=2:m-1                            
   h1=FID(i)-1;
   h2=FID(i);
   h3=FID(i)+1;
   CZ(s)=h1;
   CZ(s+1)=h2;
   CZ(s+2)=h3;
   s=i*3;  
end                                    
max1=max(CZ);
min1=min(CZ);
CZ(CZ<1)=1;
CZ(CZ>m1)=m1;
max2=max(CZ);
min2=min(CZ);
CZ1=flow1(CZ,:);                      
K=zeros(m,1,'double');
K(1)=(CZ1(2,3)-CZ1(1,3))/(CZ1(2,2)-CZ1(1,2));
K(m)=(CZ1(n,3)-CZ1(n-1,3))/(CZ1(n,2)-CZ1(n-1,2));
for i=2:m-1
   y2=CZ1(3*i-1,3);
   y1=CZ1(3*i-3,3);
   x2=CZ1(3*i-1,2);
   x1=CZ1(3*i-3,2);
   K(i)=(y2-y1)/(x2-x1);               % k is the slope of the line segment composed of the left and right discrete points of the virtual site
end
K1=-1./K;                              % k1 is the slope of the section where the river width located
ZCL=zeros(m,4,'double');
ZCL(:,1)=flow1(FID,2);
ZCL(:,2)=flow1(FID,3);
ZCL(:,3)=K1;                          
b1 = ZCL(:,2)-ZCL(:,3).*ZCL(:,1);
ZCL(:,4)=b1;                           
d=500;                                 % d is the maximum distance to retrieve outwards. The base value is set to 500m 
b1=ZCL(:,4);
x1=ZCL(:,1);                           % x1 is the abscissa of the virtual hydrological station
y1=ZCL(:,2);                           % y1 is the ordinate of the virtual hydrological station
a=K1.^2+1;
b=2*(K1.*b1-x1-K1.*y1);
c=y1.^2+x1.^2+b1.^2-2*b1.*y1-d^2;
deta1=b.^2-4*a.*c;
deta=sqrt(b.^2-4*a.*c);
x3=(-b+deta)./(2*a);                   % x3 is the abscissa of the discrete points of the center line on the right side of the virtual hydrological station
x4=(-b-deta)./(2*a);                   % x4 is the abscissa of the discrete points of the center line on the left side
y3=K1.*x3+b1;
y4=K1.*x4+b1;
ZB=zeros(m,9,'double');                
% Store sequential left point of sample - sample point -  right point of sample
% num will be the index number for the river width
num=1:m;
ZB(:,1)=num;
ZB(:,2)=x4;
ZB(:,3)=y4;
ZB(:,4)=num;
ZB(:,5)=x1;
ZB(:,6)=y1;
ZB(:,7)=num;
ZB(:,8)=x3;
ZB(:,9)=y3;
mm=3*m;
ZBX=zeros(m,3,'double');
ZBY=zeros(m,3,'double');
ZB1=zeros(3*m,3,'double');
ZBX(:,1)=x4;
ZBX(:,2)=x1;
ZBX(:,3)=x3;
ZBY(:,1)=y4;
ZBY(:,2)=y1;
ZBY(:,3)=y3;
ZBX=reshape(ZBX,3*m,1);
ZBY=reshape(ZBY,3*m,1);
ZB1(1:m,1)=num;
ZB1(m+1:2*m,1)=num;
ZB1(2*m+1:3*m,1)=num;
ZB1(:,2)=ZBX;
ZB1(:,3)=ZBY;
dlmwrite('\',ZB1,'delimiter',',','precision','%12.5f');
fileID=fopen('\','a+');
fprintf(fileID,'\n %6s %12s\n','Coordinates','WGS1984-UTM XX N');
fclose(fileID);
%% step 5 This module is mainly used to supplement the width of missing or wrong river widths
% The work of locating error point of river width is carried out on Arcgis software platform
flow1=xlsread('\',1); % Read the coordinates of discrete points of the river skeleton line in the format of X and Y and exist in Sheet1.As same as step4
flow2=xlsread('\',3); % Read error data in format X, Y and stored in Sheet2.
[m1,~]=size(flow1); 
[m2,n2]=size(flow2); 
F1 = flow1(:,4:5);   
F2 = flow2(:,4:5);
X=zeros(m1,1,'double');
l=0;
for i = 1:m2                           
  A = F2(i,1);
  [x,~] = find(F1(:,1) ==A );
  [K1,~]=size(x);
  z=l+K1;
  w=1;
  for j=l+1:z
      X(j)=x(w);
      w=w+1;
  end
   l=sum(X~=0);
end                                    
X(X==0)=[];                            
[K1,~]=size(X);
X1=zeros(K1,3,'double');
X1(:,1)=X;
X1(:,2)=F1(X,1);
X1(:,3)=F1(X,2);
X2=zeros(m1,1,'double');
l=0;
for i = 1:m2                           
  A = F2(i,2);
  [x,~] = find(X1(:,3) ==A );
  [K1,~]=size(x);
  z=l+K1;
  w=1;
  for j=l+1:z
      X2(j)=x(w);
      w=w+1;
  end
   l=sum(X2~=0);
end                                    
X2(X2==0)=[];                          
FID=X1(X2,1);                          
KK=F1(FID,:);                          
c1=ismember(KK,F2,'rows');   
[c2,~]=find(c1==0);
FID(c2,:)=[];   
FID=unique(FID);
beg=max(FID);                          
en=min(FID);
[m,~]=size(FID);
n=(m-2)*2+(m-2)+2*2;                  
CZ=zeros(n,1,'double');
CZ(1)=en;
CZ(2)=en+1;
CZ(n)=beg;
CZ(n-1)=beg-1;                         
s=3;
for i=2:m-1                            
   h1=FID(i)-1;
   h2=FID(i);
   h3=FID(i)+1;
   CZ(s)=h1;
   CZ(s+1)=h2;
   CZ(s+2)=h3;
   s=i*3;  
end                                   
max1=max(CZ);
min1=min(CZ);
CZ(CZ<1)=1;
CZ(CZ>m1)=m1;
max2=max(CZ);
min2=min(CZ);
CZ1=flow1(CZ,:);                      
K=zeros(m,1,'double');
K(1)=(CZ1(2,3)-CZ1(1,3))/(CZ1(2,2)-CZ1(1,2));
K(m)=(CZ1(n,3)-CZ1(n-1,3))/(CZ1(n,2)-CZ1(n-1,2));
for i=2:m-1
   y2=CZ1(3*i-1,3);
   y1=CZ1(3*i-3,3);
   x2=CZ1(3*i-1,2);
   x1=CZ1(3*i-3,2);
   K(i)=(y2-y1)/(x2-x1);   
end
K1=-1./K;                             
ZCL=zeros(m,4,'double');
ZCL(:,1)=flow1(FID,2);
ZCL(:,2)=flow1(FID,3);
ZCL(:,3)=K1;                          
b1 = ZCL(:,2)-ZCL(:,3).*ZCL(:,1);
ZCL(:,4)=b1;                           
b1=ZCL(:,4);
x1=ZCL(:,1);
y1=ZCL(:,2);
a=K1.^2+1;
b=2*(K1.*b1-x1-K1.*y1);
% On the basis of 500 meters retrieval river width, the retrieval width is increased to 2500m by 50% step, and calculating the final river width
d=500;                                
d1=d*0.5;                     % d1 is the extension step.
d2=d*5.0;                     % d2 is the maximum extension width.
dd=linspace(d+d1,d2,10);      % dd stores each new search width from 550 to 2500m. it can be adjusted by changing d1 and d2.       
[~,j]=size(dd);
c=zeros(m,j,'double');
deta1=zeros(m,j,'double');
deta=zeros(m,j,'double');
for i=1:j
   c(:,i)= y1.^2+x1.^2+b1.^2-2*b1.*y1-dd(i)^2;
   deta1(:,i)=b.^2-4*a.*c(:,i);
   deta(:,i)=sqrt(b.^2-4*a.*c(:,i));
end    
zuobiao=zeros(m,7,j);
zb_x3=zeros(m,j);
zb_y3=zeros(m,j);
zb_x4=zeros(m,j);
zb_y4=zeros(m,j);
num=1:m;
for i=1:j
  zb_x3(:,i)=(-b+deta(:,i))./(2*a);        % x1,x3,x4 mean the same thing in step4
  zb_x4(:,i)=(-b-deta(:,i))./(2*a);
  zb_y3(:,i)=K1.*zb_x3(:,i)+b1;
  zb_y4(:,i)=K1.*zb_x4(:,i)+b1;
  zuobiao(:,1,i)=num;
  zuobiao(:,2,i)=num;
  zuobiao(:,3,i)=num;
  zuobiao(:,4,i)=zb_x4(:,i);
  zuobiao(:,5,i)=x1;
  zuobiao(:,6,i)=zb_x3(:,i);
  zuobiao(:,7,i)=zb_y4(:,i);
  zuobiao(:,8,i)=y1;
  zuobiao(:,9,i)=zb_y3(:,i);
end
zuobiao1=reshape(zuobiao,3*m,3*j);
dlmwrite('\',zuobiao1,'delimiter',',','precision','%12.5f');
fileID=fopen('\','a+');
fprintf(fileID,'\n %6s %12s\n');
fclose(fileID);
% The identified missing river width is added to the original data 
soucedata=xlsread('\',1); 
Add_data=xlsread('\',2); 
[ad,~]=size(Add_data);
for i=1:ad
   [ad1,~]=find(soucedata(:,1)==Add_data(i,1));
   soucedata(ad1,2)=Add_data(i,2);
end
[sd,~]=find(soucedata(:,2)==1000);
soucedata(sd,:)=[];
dlmwrite('\',soucedata,'delimiter',',','precision','%12.5f');       % Can use ArcGis software platform to check the new river width .